#include "GLWindow.h"
#include <iostream>
#include "ngl/Camera.h"
#include "ngl/Colour.h"
#include "ngl/Light.h"
#include "ngl/Matrix.h"
#include "ngl/Transformation.h"
#include "ngl/TransformStack.h"
#include "ngl/Material.h"
#include "ngl/NGLInit.h"
#include "ngl/Obj.h"
#include "ngl/VBOPrimitives.h"
#include "ngl/ShaderManager.h"
#include "ngl/Util.h"
#include "ngl/Random.h"
#include <boost/foreach.hpp>
#include "ngl/Util.h"
#include <ctime>


//----------------------------------------------------------------------------------------------------------------------
GLWindow::GLWindow(
                   QWidget *_parent
                  ) :
                    QGLWidget(_parent)
{

  // set this widget to have the initial keyboard focus
  setFocus();
  // re-size the widget to that of the parent (in this case the GLFrame passed in on construction)
  this->resize(_parent->size());
  // Now set the initial GLWindow attributes to default values
  // Roate is false
  m_rotate=false;
  // mouse rotation values set to 0
  m_spinXFace=0;
  m_spinYFace=0;
  m_origX=0;
  m_origY=0;
}

GLWindow::~GLWindow()
{
  ngl::NGLInit *Init = ngl::NGLInit::instance();
  std::cout<<"Shutting down NGL, removing VBO's and Shaders\n";
  Init->NGLQuit();
}

// This virtual function is called once before the first call to paintGL() or resizeGL(),
//and then once whenever the widget has been assigned a new QGLContext.
// This function should set up any required OpenGL context rendering flags, defining display lists, etc.

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::initializeGL()
{

  glClearColor(0.4f, 0.4f, 0.4f, 1.0f);			   // Grey Background
  // enable depth testing for drawing
   glEnable(GL_DEPTH_TEST);
   // Now we will create a basic Camera from the graphics library
   // This is a static camera so it only needs to be set once
   // First create Values for the camera position
   ngl::Vector From(4,4,2);
   ngl::Vector To(0,0,0);
   ngl::Vector Up(0,1,0);
   ngl::NGLInit *Init = ngl::NGLInit::instance();
   #ifdef WIN32
   	glewInit();
   #endif
   Init->initGlew();
   m_cam= new ngl::Camera(From,To,Up,ngl::PERSPECTIVE);
   // set the shape using FOV 45 Aspect Ratio based on Width and Height
   // The final two are near and far clipping planes of 0.5 and 10
   m_cam->setShape(45,(float)720.0/576.0,0.5,150,ngl::PERSPECTIVE);
   // now to load the shader and set the values
   // grab an instance of shader manager
   ngl::ShaderManager *shader=ngl::ShaderManager::instance();
   // load a frag and vert shaders
   shader->loadShader("Blinn","shaders/Vertex.vs","shaders/Fragment.fs");
   // set this as the active shader
   shader->useShader("Blinn");
   // now pass the modelView and projection values to the shader
   shader->setShaderParamFromMatrix("Blinn","ViewMatrix",m_cam->getModelView());
   shader->setShaderParamFromMatrix("Blinn","projectionMatrix",m_cam->getProjection());

   glEnable(GL_DEPTH_TEST); // for removal of hidden surfaces
	 // first we create a mesh from an obj passing in the obj file and texture
	 m_mesh = new ngl::Obj("models/Helix.obj","textures/helix_base.tif");

	 // now we need to create this as a VBO so we can draw it
	 m_mesh->createVBO(GL_STATIC_DRAW);
	 m_mesh->calcBoundingSphere();
	 ngl::VBOPrimitives *prim=ngl::VBOPrimitives::instance();
	 prim->createVBOSphere("sphere",1.0,20);
	 glEnable(GL_MULTISAMPLE);
}

//----------------------------------------------------------------------------------------------------------------------
//This virtual function is called whenever the widget has been resized.
// The new size is passed in width and height.
void GLWindow::resizeGL(
                        int _w,
                        int _h
                       )
{
  glViewport(0,0,_w,_h);
  m_cam->setShape(45,(float)_w/_h,0.5,150,ngl::PERSPECTIVE);
  ngl::ShaderManager *shader=ngl::ShaderManager::instance();
  shader->setShaderParamFromMatrix("Blinn","projectionMatrix",m_cam->getProjection());
}

//----------------------------------------------------------------------------------------------------------------------
//This virtual function is called whenever the widget needs to be painted.
// this is our main drawing routine
void GLWindow::paintGL()
{
  // clear the screen and depth buffer
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 // build our transform stack
  ngl::Transformation trans;
  // set the mouse rotation
  trans.setRotation(m_spinXFace,m_spinYFace,0);
  // set this in the TX stack
  m_transformStack.setGlobal(trans);

  ngl::ShaderManager *shader=ngl::ShaderManager::instance();
  m_transformStack.loadGlobalAndCurrentMatrixToShader("Blinn","ModelMatrix");
  shader->setShaderParamFromMatrix("Blinn","ModelMatrix",m_transformStack.getCurrAndGlobal().getMatrix());
	// draw the mesh
	m_mesh->draw();
	// draw the mesh bounding box
	m_mesh->drawBBox();

	ngl::VBOPrimitives *prim=ngl::VBOPrimitives::instance();

	m_transformStack.pushTransform();
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
			m_transformStack.getCurrentTransform().setPosition(m_mesh->getSphereCenter());
			m_transformStack.getCurrentTransform().setScale(m_mesh->getSphereRadius(),m_mesh->getSphereRadius(),m_mesh->getSphereRadius());

			m_transformStack.loadGlobalAndCurrentMatrixToShader("Blinn","ModelMatrix");
			prim->draw("sphere");
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

	m_transformStack.popTransform();

}

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mouseMoveEvent (
                               QMouseEvent * _event
                              )
{
  // note the method buttons() is the button state when event was called
  // this is different from button() which is used to check which button was
  // pressed when the mousePress/Release event is generated
  if(m_rotate && _event->buttons() == Qt::LeftButton)
  {
    m_spinYFace = ( m_spinYFace + (_event->x() - m_origX) ) % 360 ;
    m_spinXFace = ( m_spinXFace + (_event->y() - m_origY) ) % 360 ;
    m_origX = _event->x();
    m_origY = _event->y();
  }
  // re-draw GL
  updateGL();
}


//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mousePressEvent (
                                QMouseEvent * _event
                               )
{
  // this method is called when the mouse button is pressed in this case we
  // store the value where the maouse was clicked (x,y) and set the Rotate flag to true
  if(_event->button() == Qt::LeftButton)
  {
    m_origX = _event->x();
    m_origY = _event->y();
    m_rotate =true;
  }
}

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mouseReleaseEvent (
                                  QMouseEvent * _event
                                 )
{
  // this event is called when the mouse button is released
  // we then set Rotate to false
  if (_event->button() == Qt::LeftButton)
  {
    m_rotate=false;
  }
}

void GLWindow::timerEvent(
                          QTimerEvent *_event
                         )
{

}

